<?php
/**
 * PHP Wrapper Example
 *
 * @author Benny Baumann (BenBE) <BenBE@geshi.org>
 * @version $Id$
 * @copyright 2009-2012 by Benny Baumann
 */


require 'rrdtool.php';

function domainToID($domain) {
    if('ALL' == $domain) {
        return 'ALL_DATA';
    }

    return sprintf('%8s', substr(md5($domain), 0, 8));
}

function addDSbyName($match, $ds) {
    $dsmatch = array_filter($ds, function($a) use ($match) { return preg_match($match, $a); } );
    return count($dsmatch) ? implode(',', $dsmatch) . str_repeat(',+', count($dsmatch) - 1) : '0,0,+';
}

//Get all the domains we know about
//header('Content-Type: text/plain; charset="utf-8"');
$domains = scandir('rrd');
$domains = array_filter($domains, function($a) { return preg_match('/\.rrd$/', $a); } );
array_walk($domains, function(&$a) { $a = str_replace('.rrd', '', $a); return true; } );

$d = !isset($_GET['d']) ? 'ALL' : $_GET['d'];
if(in_array($d, $domains)) {
    $domains = array($d);
}

//Load all rrd files and generate DEF and CDF statements on the fly
$rrd = array();
$rrdDS = array();
foreach($domains as $domain) {
    $rrd[$domain] = RRDTool::Archive();
    $rrd[$domain]->Load("rrd/$domain.rrd");
    $DS = $rrd[$domain]->Info();
    foreach($DS['ds'] as $dsname => $ds) {
        $rrdDS['r' . domainToID($domain) . '_' . $dsname] = sprintf('DEF:r%s=rrd/%s.rrd:%s:AVERAGE', domainToID($domain) . '_' . $dsname, $domain, $dsname);
        $rrdDS['c' . domainToID($domain) . '_' . $dsname] = sprintf('CDEF:c%1$s=r%1$s,UN,0,r%1$s,IF', domainToID($domain) . '_' . $dsname);
    }
}

//Assume all files have the same layout; generate CDEF for the ALL domain
$dsnames = array_keys($DS['ds']);
foreach($dsnames as $dsname) {
    //Now let's do some black magic:
    $rrdDS['c' . domainToID('ALL') . '_' . $dsname] = sprintf(
        'CDEF:c%s=%s', 
        domainToID('ALL') . '_' . $dsname, 
        addDSbyName('/^c.{8}_'.preg_quote($dsname).'$/', array_keys($rrdDS)));
}

//echo "<pre>";
//var_dump($rrdDS);
//echo "</pre>";

if(isset($_GET['g']) && isset($_GET['d'])) {
    $g = $_GET['g'];
    $d = $_GET['d'];

    $g = preg_replace('/[^\w\-\.]/', '', $g);
    $d = preg_replace('/[^\w\-\.]/', '', $d);

    //build the graph
    $draw = array();
    switch($g)
    {
        case 'io':
            $draw[] = 'VDEF:vAVG_'.(domainToID($d).'_trafficv4').'=c'.(domainToID($d).'_trafficv4').',AVERAGE';
            $draw[] = 'VDEF:vMIN_'.(domainToID($d).'_trafficv4').'=c'.(domainToID($d).'_trafficv4').',MINIMUM';
            $draw[] = 'VDEF:vMAX_'.(domainToID($d).'_trafficv4').'=c'.(domainToID($d).'_trafficv4').',MAXIMUM';

            $draw[] = 'VDEF:vAVG_'.(domainToID($d).'_trafficv6').'=c'.(domainToID($d).'_trafficv6').',AVERAGE';
            $draw[] = 'VDEF:vMIN_'.(domainToID($d).'_trafficv6').'=c'.(domainToID($d).'_trafficv6').',MINIMUM';
            $draw[] = 'VDEF:vMAX_'.(domainToID($d).'_trafficv6').'=c'.(domainToID($d).'_trafficv6').',MAXIMUM';

            $draw[] = 'COMMENT:         Average         Minimum         Maximum\l';

            $draw[] = 'AREA:c'.(domainToID($d).'_trafficv4').'#00CC88:IPv4';
            $draw[] = 'GPRINT:vAVG_'.(domainToID($d).'_trafficv4').':%9.2lf %SB/s';
            $draw[] = 'GPRINT:vMIN_'.(domainToID($d).'_trafficv4').':%9.2lf %SB/s';
            $draw[] = 'GPRINT:vMAX_'.(domainToID($d).'_trafficv4').':%9.2lf %SB/s\l';

            $draw[] = 'AREA:c'.(domainToID($d).'_trafficv6').'#00CC00:IPv6:STACK';
            $draw[] = 'GPRINT:vAVG_'.(domainToID($d).'_trafficv6').':%9.2lf %SB/s';
            $draw[] = 'GPRINT:vMIN_'.(domainToID($d).'_trafficv6').':%9.2lf %SB/s';
            $draw[] = 'GPRINT:vMAX_'.(domainToID($d).'_trafficv6').':%9.2lf %SB/s\l';

            $draw[] = 'LINE:c'.(domainToID($d).'_traffic').'#000000';
            break;
        case 'ip':
            $draw[] = 'VDEF:vAVG_'.(domainToID($d).'_ipv4').'=c'.(domainToID($d).'_ipv4').',AVERAGE';
            $draw[] = 'VDEF:vMIN_'.(domainToID($d).'_ipv4').'=c'.(domainToID($d).'_ipv4').',MINIMUM';
            $draw[] = 'VDEF:vMAX_'.(domainToID($d).'_ipv4').'=c'.(domainToID($d).'_ipv4').',MAXIMUM';

            $draw[] = 'VDEF:vAVG_'.(domainToID($d).'_ipv6').'=c'.(domainToID($d).'_ipv6').',AVERAGE';
            $draw[] = 'VDEF:vMIN_'.(domainToID($d).'_ipv6').'=c'.(domainToID($d).'_ipv6').',MINIMUM';
            $draw[] = 'VDEF:vMAX_'.(domainToID($d).'_ipv6').'=c'.(domainToID($d).'_ipv6').',MAXIMUM';

            $draw[] = 'COMMENT:         Average         Minimum         Maximum\l';

            $draw[] = 'AREA:c'.(domainToID($d).'_ipv4').'#00CC88:IPv4';
            $draw[] = 'GPRINT:vAVG_'.(domainToID($d).'_ipv4').':%9.2lf %Sreq/s';
            $draw[] = 'GPRINT:vMIN_'.(domainToID($d).'_ipv4').':%9.2lf %Sreq/s';
            $draw[] = 'GPRINT:vMAX_'.(domainToID($d).'_ipv4').':%9.2lf %Sreq/s\l';

            $draw[] = 'AREA:c'.(domainToID($d).'_ipv6').'#00CC00:IPv6:STACK';
            $draw[] = 'GPRINT:vAVG_'.(domainToID($d).'_ipv6').':%9.2lf %Sreq/s';
            $draw[] = 'GPRINT:vMIN_'.(domainToID($d).'_ipv6').':%9.2lf %Sreq/s';
            $draw[] = 'GPRINT:vMAX_'.(domainToID($d).'_ipv6').':%9.2lf %Sreq/s\l';

            break;
        case 'http':
            $draw[] = 'AREA:c'.(domainToID($d).'_http11').'#00CC88:HTTP 1.1\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_http10').'#00CC00:HTTP 1.0\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_http09').'#CCCC00:HTTP 0.9\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_httpother').'#CC0000:other\l:STACK';
            break;
        case 'port':
            $draw[] = 'AREA:c'.(domainToID($d).'_port80').'#CC0000:Plaintext\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_port443').'#00CC00:Encrypted\l:STACK';
            break;
        case 'req':

            $draw[] = 'AREA:c'.(domainToID($d).'_reqGET').'#00CC00:GET\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqHEAD').'#00AA00:HEAD\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqOPTION').'#008800:OPTION\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqREPORT').'#004400:REPORT\l:STACK';

            $draw[] = 'AREA:c'.(domainToID($d).'_reqPOST').'#CCCC00:POST\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqMERGE').'#CCAA00:MERGE\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqPUT').'#CC8800:PUT\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqDELETE').'#CC4400:DELETE\l:STACK';

            $draw[] = 'AREA:c'.(domainToID($d).'_reqMKCOL').'#CC00CC:MKCOL\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqPROPFIND').'#8800CC:PROPFIND\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqPROPPATCH').'#CC0088:PROPPATCH\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_reqCOPY').'#CC0044:COPY\l:STACK';

            $draw[] = 'AREA:c'.(domainToID($d).'_reqCONNECT').'#CC0000:CONNECT\l:STACK';

            $draw[] = 'AREA:c'.(domainToID($d).'_reqother').'#888888:other\l:STACK';

            break;
        case 'status':
            $draw[] = 'CDEF:t'.(domainToID($d).'_status100').'='.addDSbyName('/^c'.preg_quote(domainToID($d), '/').'_status1\d\d$/', array_keys($rrdDS));
            $draw[] = 'CDEF:t'.(domainToID($d).'_status200').'='.addDSbyName('/^c'.preg_quote(domainToID($d), '/').'_status2\d\d$/', array_keys($rrdDS));
            $draw[] = 'CDEF:t'.(domainToID($d).'_status300').'='.addDSbyName('/^c'.preg_quote(domainToID($d), '/').'_status3\d\d$/', array_keys($rrdDS));
            $draw[] = 'CDEF:t'.(domainToID($d).'_status400').'='.addDSbyName('/^c'.preg_quote(domainToID($d), '/').'_status4\d\d$/', array_keys($rrdDS));
            $draw[] = 'CDEF:t'.(domainToID($d).'_status500').'='.addDSbyName('/^c'.preg_quote(domainToID($d), '/').'_status5\d\d$/', array_keys($rrdDS));

            $draw[] = 'AREA:t'.(domainToID($d).'_status100').'#00CCCC:Status 1xx';
            $draw[] = 'AREA:t'.(domainToID($d).'_status200').'#00CC00:Status 2xx:STACK';
            $draw[] = 'AREA:t'.(domainToID($d).'_status300').'#CCCC00:Status 3xx:STACK';
            $draw[] = 'AREA:t'.(domainToID($d).'_status400').'#CC0000:Status 4xx:STACK';
            $draw[] = 'AREA:t'.(domainToID($d).'_status500').'#CC00CC:Status 5xx:STACK';

            break;
        case 'status_detail':
            $codes = array(
                'Status 1xx (Informational)' => array(
                    array(100, '#00CCCC', 'Continue'),
                    array(101, '#00BBBB', 'Switching Protocols'),
                    array(102, '#00AAAA', 'Processing'),
                    array(103, '#009999', 'Checkpoint'),
                    array(122, '#008888', 'Request-URI too long'),
                    ),
                'Status 2xx (Success)' => array(
                    array(200, '#00CC00', 'Ok'),
                    array(201, '#00BB00', 'Created'),
                    array(202, '#00AA00', 'Accepted'),
                    array(203, '#009900', 'Non-authrotive information'),
                    array(204, '#008800', 'No Content'),
                    array(205, '#007700', 'Reset Content'),
                    array(206, '#006600', 'Partial Content'),
                    array(207, '#005500', 'Multi-Status'),
                    array(208, '#004400', 'Already Reported'),
                    array(226, '#003300', 'IM Used'),
                    ),
                'Status 3xx (Redirection)' => array(
                    array(300, '#CCCC00', 'Multiple Choices'),
                    array(301, '#BBBB00', 'Moved Permanently'),
                    array(302, '#AAAA00', 'Found'),
                    array(303, '#999900', 'See Other'),
                    array(304, '#888800', 'Not Modified'),
                    array(305, '#777700', 'Use Proxy'),
                    array(306, '#666600', 'Switch Proxy'),
                    array(307, '#555500', 'Temporary Redirect'),
                    array(308, '#444400', 'Resume Incomplete'),
                    ),
                'Status 4xx (Client Error)' => array(
                    array(400, '#FF0000', 'Bad Request'),
                    array(401, '#F70000', 'Unauthorized'),
                    array(402, '#EF0000', 'Payment Required'),
                    array(403, '#E70000', 'Forbidden'),
                    array(404, '#DF0000', 'Not found'),
                    array(405, '#D70000', 'Method Not Allowed'),
                    array(406, '#CF0000', 'Not Acceptable'),
                    array(407, '#C70000', 'Proxy Authentication Required'),
                    array(408, '#BF0000', 'Request Timeout'),
                    array(409, '#B70000', 'Conflict'),
                    array(410, '#AF0000', 'Gone'),
                    array(411, '#A70000', 'Length Required'),
                    array(412, '#9F0000', 'Precondition Failed'),
                    array(413, '#970000', 'Request Entity Too Large'),
                    array(414, '#8F0000', 'Request URI too Long'),
                    array(415, '#870000', 'Unsupported Media Type'),
                    array(416, '#7F0000', 'Request Range Not Satisfiable'),
                    array(417, '#770000', 'Expectation Failed'),
                    array(418, '#6F0000', 'I\'m A Teapot'),
                    array(420, '#670000', 'Enhance Your Calm'),
                    array(422, '#5F0000', 'Unprocessable Entity'),
                    array(423, '#570000', 'Locked'),
                    array(424, '#4F0000', 'Failed Dependency'),
                    array(425, '#470000', 'Unordered Collection'),
                    array(426, '#3F0000', 'Upgrade Required'),
                    array(428, '#370000', 'Precondition Required'),
                    array(429, '#2F0000', 'Too Many Requests'),
                    array(431, '#270000', 'Request Header Fields Too Large'),
                    array(444, '#1F0000', 'No Response'),
                    array(449, '#170000', 'Retry With'),
                    array(450, '#0F0000', 'Blocked By Parental Control'),
                    array(499, '#070000', 'Client Closed Request'),
                    ),
                'Status 5xx (Server Error)' => array(
                    array(500, '#FF00FF', 'Internal Server Error'),
                    array(501, '#EE00EE', 'Not Implemented'),
                    array(502, '#DD00DD', 'Bad Gateway'),
                    array(503, '#CC00CC', 'Service Unavailable'),
                    array(504, '#BB00BB', 'Gateway Timeout'),
                    array(505, '#AA00AA', 'HTTP Version Not Supported'),
                    array(506, '#990099', 'Variant Also Negotiates'),
                    array(507, '#880088', 'Insufficient Storage'),
                    array(508, '#770077', 'Loop Detected'),
                    array(509, '#660066', 'Bandwidth Limit Exceeded'),
                    array(510, '#550055', 'Not Extended'),
                    array(511, '#440044', 'Network Authentication Required'),
                    array(598, '#330033', 'Network Read Timeout'),
                    array(599, '#220022', 'Network Connect Timeout'),
                    )
                );

            $draw[] = sprintf('COMMENT:  Status    %-32s %18s%18s%18s\l', '', 'Minimum', 'Maximum', 'Average');
            $draw[] = 'LINE:0';
            foreach($codes as $groupname => $groupdata) {
                $draw[] = 'COMMENT:  '.$groupname.'\l';
                foreach($groupdata as $groupentry) {
                    list($statuscode, $statuscolor, $statusname) = $groupentry;

                    $draw[] = sprintf('VDEF:vMIN_%s_status%03d=c%s_status%03d,MINIMUM',
                        domainToID($d), $statuscode, domainToID($d), $statuscode);
                    $draw[] = sprintf('VDEF:vMAX_%s_status%03d=c%s_status%03d,MAXIMUM',
                        domainToID($d), $statuscode, domainToID($d), $statuscode);
                    $draw[] = sprintf('VDEF:vAVG_%s_status%03d=c%s_status%03d,AVERAGE',
                        domainToID($d), $statuscode, domainToID($d), $statuscode);

                    $draw[] = sprintf('AREA:c%s_status%03d%7s:Status %03d %-32s:STACK',
                        domainToID($d), $statuscode, $statuscolor, $statuscode, $statusname);

                    $draw[] = sprintf('GPRINT:vMIN_%s_status%03d:%%9.2lf %%sreq/s',
                        domainToID($d), $statuscode);
                    $draw[] = sprintf('GPRINT:vMAX_%s_status%03d:%%9.2lf %%sreq/s',
                        domainToID($d), $statuscode);
                    $draw[] = sprintf('GPRINT:vAVG_%s_status%03d:%%9.2lf %%sreq/s\l',
                        domainToID($d), $statuscode);
                }
                $draw[] = 'COMMENT: \l';
            }
/*
            $draw[] = 'COMMENT:Status 1xx (Informational)\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_status100').'#00CCCC:Status 100\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status101').'#00BBBB:Status 101\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status102').'#00AAAA:Status 102\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status103').'#009999:Status 103\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status122').'#008888:Status 122\l:STACK';

            $draw[] = 'COMMENT:\l';
            $draw[] = 'COMMENT:Status 2xx (Success)\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_status200').'#00CC00:Status 200\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status201').'#00BB00:Status 201\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status202').'#00AA00:Status 202\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status203').'#009900:Status 203\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status204').'#008800:Status 204\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status205').'#007700:Status 205\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status206').'#006600:Status 206\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status207').'#005500:Status 207\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status208').'#004400:Status 208\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status226').'#003300:Status 226\l:STACK';

            $draw[] = 'COMMENT:\l';
            $draw[] = 'COMMENT:Status 3xx (Redirection)\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_status300').'#CCCC00:Status 300\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status301').'#BBBB00:Status 301\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status302').'#AAAA00:Status 302\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status303').'#999900:Status 303\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status304').'#888800:Status 304\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status305').'#777700:Status 305\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status306').'#666600:Status 306\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status307').'#555500:Status 307\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status308').'#444400:Status 308\l:STACK';

            $draw[] = 'COMMENT:\l';
            $draw[] = 'COMMENT:Status 4xx (Client Error)\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_status400').'#FF0000:Status 400\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status401').'#F70000:Status 401\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status402').'#EF0000:Status 402\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status403').'#E70000:Status 403\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status404').'#DF0000:Status 404\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status405').'#D70000:Status 405\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status406').'#CF0000:Status 406\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status407').'#C70000:Status 407\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status408').'#BF0000:Status 408\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status409').'#B70000:Status 409\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status410').'#AF0000:Status 410\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status411').'#A70000:Status 411\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status412').'#9F0000:Status 412\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status413').'#970000:Status 413\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status414').'#8F0000:Status 414\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status415').'#870000:Status 415\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status416').'#7F0000:Status 416\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status417').'#770000:Status 417\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status418').'#6F0000:Status 418\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status420').'#670000:Status 420\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status422').'#5F0000:Status 422\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status423').'#570000:Status 423\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status424').'#4F0000:Status 424\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status425').'#470000:Status 425\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status426').'#3F0000:Status 426\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status428').'#370000:Status 428\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status429').'#2F0000:Status 429\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status431').'#270000:Status 431\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status444').'#1F0000:Status 444\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status449').'#170000:Status 449\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status450').'#0F0000:Status 450\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status499').'#070000:Status 499\l:STACK';

            $draw[] = 'COMMENT:\l';
            $draw[] = 'COMMENT:Status 5xx (Server Error)\l';
            $draw[] = 'AREA:c'.(domainToID($d).'_status500').'#FF00FF:Status 500\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status501').'#EE00EE:Status 501\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status502').'#DD00DD:Status 502\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status503').'#CC00CC:Status 503\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status504').'#BB00BB:Status 504\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status505').'#AA00AA:Status 505\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status506').'#990099:Status 506\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status507').'#880088:Status 507\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status508').'#770077:Status 508\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status509').'#660066:Status 509\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status510').'#550055:Status 510\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status511').'#440044:Status 511\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status598').'#330033:Status 598\l:STACK';
            $draw[] = 'AREA:c'.(domainToID($d).'_status599').'#220022:Status 599\l:STACK';
*/
            break;
        default:
            break;
    }
    if(count($draw)) {
        $cmd = array();
        $cmd[] = "--end=now";
        $cmd[] = "--start=end-365d";

        $cmd[] = "--width";
        $cmd[] = "1024";

        $cmd[] = "--height";
        $cmd[] = "512";

//        $cmd[] = "-X";
//        $cmd[] = "0";

        $cmd[] = "-L";
        $cmd[] = "5";

        foreach($rrdDS as $c) {
            $cmd[] = $c;
        }
        foreach($draw as $c) {
            $cmd[] = $c;
        }
//        var_dump($cmd);
        if(@filemtime('png/' . $g . '_' . $d . '.png') < time() - 3600) {
            rrd_graph('png/' . $g . '_' . $d . '.png', $cmd);
            echo rrd_error();
        }
    }

    switch($g)
    {
        case 'io':
        case 'ip':
        case 'http':
        case 'port':
        case 'req':
        case 'status':
        case 'status_detail':
            header('Content-Type: image/png');
            echo file_get_contents('png/' . $g . '_' . $d . '.png');
        default:
            header('Status: 404 Not found');
            header('Content-Type: text/plain');
            echo "Error in request for graph type $g for domain $d.";
    }

    exit;
}

?>
<html>
<head>
<title>Various graphs based on the server logs</title>
</head>
<body>
<?php
if(1 < count($domains))
echo <<<HTML
    <h1>Statistics for all domains</h1>

    <h2>Traffic statistics</h2>
    <img src="?g=io&amp;d=ALL" alt="Traffic stats" /><br />

    <h2>IPv4 vs IPv6 usage stats</h2>
    <img src="?g=ip&amp;d=ALL" alt="IPv4/IPv6 stats" /><br />

    <h2>HTTP protocol version stats</h2>
    <img src="?g=http&amp;d=ALL" alt="HTTP protocol version stats" /><br />

    <h2>Use of encryption</h2>
    <img src="?g=port&amp;d=ALL" alt="Plain/SSL request stats" /><br />

    <h2>HTTP request method stats</h2>
    <img src="?g=req&amp;d=ALL" alt="Requst method stats" /><br />

    <h2>HTTP status code stats</h2>
    <img src="?g=status&amp;d=ALL" alt="Request status stats (simple)" /><br />
    <img src="?g=status_detail&amp;d=ALL" alt="Request status stats (detailled)" /><br />

HTML;

foreach($domains as $domain) {
    echo <<<HTML
    <h1>Statistics for $domain</h1>

    <h2>Traffic statistics</h2>
    <img src="?g=io&amp;d=$domain" alt="Traffic stats" /><br />

    <h2>IPv4 vs IPv6 usage stats</h2>
    <img src="?g=ip&amp;d=$domain" alt="IPv4/IPv6 stats" /><br />

    <h2>HTTP protocol version stats</h2>
    <img src="?g=http&amp;d=$domain" alt="HTTP protocol version stats" /><br />

    <h2>Use of encryption</h2>
    <img src="?g=port&amp;d=$domain" alt="Plain/SSL request stats" /><br />

    <h2>HTTP request method stats</h2>
    <img src="?g=req&amp;d=$domain" alt="Requst method stats" /><br />

    <h2>HTTP status code stats</h2>
    <img src="?g=status&amp;d=$domain" alt="Request status stats (simple)" /><br />
    <img src="?g=status_detail&amp;d=$domain" alt="Request status stats (detailled)" /><br />

HTML;

}

?>
</body>
</html>
